@@ -1,5 +1,5 @@
 ; libctl: flexible Guile-based control files for scientific software 
-; Copyright (C) 1998, 1999, 2000, 2001, 2002, Steven G. Johnson
+; Copyright (C) 1998, 1999, 2000, 2001, 2002, 2006, Steven G. Johnson
 ;
 ; This library is free software; you can redistribute it and/or
 ; modify it under the terms of the GNU Lesser General Public
@@ -42,6 +42,31 @@
      (reverse (cdr (reverse nums))) ; nums w/o last value
      (cdr nums))))) ; nums w/o first value
 
+; Like interpolate, except only interpolates n values *on average*
+; between each pair of numbers.  The actual number of interpolated
+; points varies for each pair to try to keep the density of points
+; uniform.
+(define (interpolate-uniform n nums)
+  (define meandiff
+    (/ (fold-left + 0 (map unary-abs (map binary- (cdr nums)
+					  (reverse (cdr (reverse nums))))))
+       (length (cdr nums))))
+  (if (zero? n)
+      nums
+      (cons
+       (car nums)
+       (fold-right
+	append '()
+	(map
+	 (lambda (x y)
+	   (let ((m (inexact->exact 
+		     (+ -0.5 (* (+ n 1) (/ (unary-abs (binary- x y))
+					   meandiff))))))
+	     (reverse (arith-sequence y (binary/ (binary- x y) (+ m 1)) 
+				      (+ m 1)))))
+	 (reverse (cdr (reverse nums))) ; nums w/o last value
+	 (cdr nums)))))) ; nums w/o first value
+
 ; ****************************************************************
 ; Minimization and root-finding utilities (useful in ctl scripts)
 
@@ -354,6 +379,11 @@
 	    (if (= f 0)
 		x
 		(let ((a' (if (< f 0) x a)) (b' (if (> f 0) x b)))
+		  (if (and (not (= dx (- x-max x-min)))
+			   (negative? (* dx (/ f df)))
+			   (positive? (* (car (f-memo (lazy a')))
+					 (car (f-memo (lazy b'))))))
+		      (error "failed to bracket the root in find-root-deriv"))
 		  (if (and (if (and (number? a) (number? b))
 			       (in-bounds? x f df a b)
 			       (in-bounds? x f df x-min x-max))
@@ -414,14 +444,14 @@
 
   (deriv dx 0 1e30 '() 2))
       
-(define (do-derivative-wrap f x dx-and-tol)
+(define (do-derivative-wrap do-deriv f x dx-and-tol)
   (let ((dx (if (> (length dx-and-tol) 0)
 		(car dx-and-tol)
 		(max (magnitude (* x 0.01)) 0.01)))
 	(tol (if (> (length dx-and-tol) 1)
 		 (cadr dx-and-tol)
 		 0)))
-    (do-derivative f x dx tol)))
+    (do-deriv f x dx tol)))
 
 (define derivative-df car)
 (define derivative-df-err cadr)
@@ -429,9 +459,9 @@
 (define derivative-d2f-err cadddr)
 
 (define (derivative f x . dx-and-tol)
-  (do-derivative-wrap f x dx-and-tol))
+  (do-derivative-wrap do-derivative f x dx-and-tol))
 (define (deriv f x . dx-and-tol)
-  (derivative-df (do-derivative-wrap f x dx-and-tol)))
+  (derivative-df (do-derivative-wrap do-derivative f x dx-and-tol)))
 
 ; Compute both the first and second derivatives at the same time
 ; (using minimal extra function evaluations).
@@ -440,12 +470,91 @@
   (define (f-deriv y)
     (binary* (binary- (f-memo y) (f-memo x)) (/ 2 (- y x))))
   (append
-   (do-derivative-wrap f-memo x dx-and-tol)
-   (do-derivative-wrap f-deriv x dx-and-tol)))
+   (do-derivative-wrap do-derivative f-memo x dx-and-tol)
+   (do-derivative-wrap do-derivative f-deriv x dx-and-tol)))
 
 (define (deriv2 f x . dx-and-tol)
   (derivative-d2f (apply derivative2 (cons f (cons x dx-and-tol)))))
 
+
+; Below, we have variants of the above routine which only compute the
+; *one-sided* derivative df/dx for dx > 0.  (Adapted from Ridder's
+; algorithm by SGJ.  Note that these are generally less accurate
+; than the ordinary two-sided derivative, above.)
+
+(define (do-derivative+ f x dx tol)
+
+  ; Using Neville's algorithm, compute successively higher-order
+  ; extrapolations of the derivative (the "Neville tableau"):
+  (define (deriv-a a0 prev-a fac fac0)
+    (if (null? prev-a)
+	(list a0)
+	(cons a0 (deriv-a (binary/
+			   (binary- (binary* a0 fac) (car prev-a)) 
+			   (- fac 1))
+			  (cdr prev-a) (* fac fac0) fac0))))
+  (define fx (f x))
+  
+  (define (deriv dx df0 err0 prev-a fac0)
+    (let ((a (deriv-a (binary/ (binary- (f (+ x dx)) fx) dx)
+		      prev-a fac0 fac0)))
+      (if (null? prev-a)
+	  (deriv (/ dx fac0) (car a) err0 a fac0)
+	  (let* ((errs
+		  (map max
+		       (map unary-abs (map binary- (cdr a) (reverse (cdr (reverse a)))))
+		       (map unary-abs (map binary- (cdr a) prev-a))))
+		 (errmin (apply min errs))
+		 (err (min errmin err0))
+		 (df (if (> err err0)
+			 df0
+			 (cdr (assoc errmin (map cons errs (cdr a)))))))
+	    (if (or (< err (* tol (unary-abs df)) )
+		    (> (unary-abs (binary- (car (reverse a)) (car (reverse prev-a))))
+		       (* 2 err)))
+		(list df err)
+		(deriv (/ dx fac0) df err a fac0))))))
+
+  (deriv dx 0 1e30 '() (sqrt 2)))
+      
+; Compute both the first and second derivatives at the same time
+; (using minimal extra function evaluations).
+(define (do-derivative-wrap2+ do-deriv only2? f x dx-and-tol)
+  (define f-memo (memoize f))
+  (define (f-deriv y)
+    (if (= y x) 
+	0.0
+	(binary* (binary+ (f-memo y) 
+			  (binary- (f-memo x)
+				   (binary* 2.0 (f-memo (* 0.5 (+ x y))))))
+		 (/ 4 (- y x)))))
+  (append
+   (if only2? 
+       (list 0 0)
+       (do-derivative-wrap do-deriv f-memo x dx-and-tol))
+   (do-derivative-wrap do-deriv f-deriv x dx-and-tol)))
+
+(define (derivative+ f x . dx-and-tol)
+  (do-derivative-wrap do-derivative+ f x dx-and-tol))
+(define (deriv+ f x . dx-and-tol)
+  (derivative-df (do-derivative-wrap do-derivative+ f x dx-and-tol)))
+(define (derivative2+ f x . dx-and-tol)
+  (do-derivative-wrap2+ do-derivative+ false f x dx-and-tol))
+(define (deriv2+ f x . dx-and-tol)
+  (derivative-d2f (do-derivative-wrap2+ do-derivative+ true f x dx-and-tol)))
+
+; as do-derivative+, but taking derivative from left
+(define (do-derivative- f x dx tol)
+  (do-derivative+ f x (- dx) tol))
+(define (derivative- f x . dx-and-tol)
+  (do-derivative-wrap do-derivative- f x dx-and-tol))
+(define (deriv- f x . dx-and-tol)
+  (derivative-df (do-derivative-wrap do-derivative- f x dx-and-tol)))
+(define (derivative2- f x . dx-and-tol)
+  (do-derivative-wrap2+ do-derivative- false f x dx-and-tol))
+(define (deriv2- f x . dx-and-tol)
+  (derivative-d2f (do-derivative-wrap2+ do-derivative- true f x dx-and-tol)))
+
 ; ****************************************************************
 
 ; Some simple integration routines using an adaptive trapezoidal rule
@@ -456,10 +565,13 @@
 ; Also, implementing thse simple algorithms in Scheme lets us use our
 ; polymorphic arithmetic functions so that we can easily integrate
 ; real, complex, and vector-valued functions.
+;
+; UPDATE: quadrature/cubature rules are now implemented via C
 
 ; Integrate the 1d function (f x) from x=a..b to within the specified
 ; fractional tolerance.
 (define (integrate-1d f a b tol)
+  (define (pow2 n) (if (<= n 0) 1 (* 2 (pow2 (- n 1))))) ; 2^n
   (define (trap0 n sum)
     (binary*
      0.5
@@ -488,7 +600,8 @@
 ; Integrate the multi-dimensional function f from a..b, within the
 ; specified tolerance.  a and b are either numbers (for 1d integrals),
 ; or vectors/lists of the same length giving the bounds in each dimension.
-(define (integrate f a b tol)
+;  NOTE: this is our *old* routine that uses the trapezoidal rule
+(define (integrate-old f a b tol)
   (define (int f a b)
     (if (null? a)
 	(f)
@@ -497,9 +610,25 @@
 	 (car a) (car b) tol)))
   (cond
    ((and (vector? a) (vector? b))
-    (integrate f (vector->list a) (vector->list b) tol))
+    (integrate-old f (vector->list a) (vector->list b) tol))
    ((and (number? a) (number? b))
-    (integrate f (list a) (list b) tol))
+    (integrate-old f (list a) (list b) tol))
    (else (int f a b))))
 
+; As above, but use adaptive cubature rules in integrator.c
+; Optionally, can take absolute tolerance and max # function evals as args.
+(define (integrate f a b reltol . abstol-and-maxnfe)
+  (define (to-list x)
+    (cond ((number? x) (list x))
+	  ((vector? x) (vector->list x))
+	  (else x)))
+  ((if (defined? 'cadaptive-integration) 
+       cadaptive-integration ; only compiled when complex nums are available
+       adaptive-integration)
+   (lambda (x) (apply f x)) 
+   (to-list a) (to-list b) 
+   (if (null? abstol-and-maxnfe) 0.0 (car abstol-and-maxnfe))
+   reltol
+   (if (< (length abstol-and-maxnfe) 2) 0 (cadr abstol-and-maxnfe))))
+
 ; ****************************************************************
