@@ -1,5 +1,5 @@
 ; libctl: flexible Guile-based control files for scientific software 
-; Copyright (C) 1998, 1999, 2000, 2001, 2002, Steven G. Johnson
+; Copyright (C) 1998, 1999, 2000, 2001, 2002, 2006, Steven G. Johnson
 ;
 ; This library is free software; you can redistribute it and/or
 ; modify it under the terms of the GNU Lesser General Public
@@ -25,18 +25,16 @@
 (define false #f)
 
 (define (list-transform-positive l pred)
-  (if (null? l)
-      l
-      (if (pred (car l))
-	  (cons (car l) (list-transform-positive (cdr l) pred))
-	  (list-transform-positive (cdr l) pred))))
+  (define (tp Lrest Lpos)
+    (if (null? Lrest)
+	Lpos
+	(if (pred (car Lrest))
+	    (tp (cdr Lrest) (cons (car Lrest) Lpos))
+	    (tp (cdr Lrest) Lpos))))
+  (reverse (tp l '())))
 
 (define (list-transform-negative l pred)
-  (if (null? l)
-      l
-      (if (not (pred (car l)))
-	  (cons (car l) (list-transform-negative (cdr l) pred))
-	  (list-transform-negative (cdr l) pred))))
+  (list-transform-positive l (lambda (x) (not (pred x)))))
 
 (define (alist-copy al)
   (if (null? al) '()
@@ -106,7 +104,10 @@
 (define print-ok? true) ; so that the user can disable output
 
 (define (print . items)
-  (if print-ok? (for-each (lambda (item) (display item)) items)))
+  (if print-ok? 
+      (begin
+	(for-each (lambda (item) (display item)) items)
+	(flush-all-ports))))
 
 (define display-many print) ; backwards compatibility with earlier libctl
 
