@@ -1,5 +1,5 @@
 /* libctl: flexible Guile-based control files for scientific software 
- * Copyright (C) 1998, 1999, 2000, 2001, 2002, Steven G. Johnson
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2006, Steven G. Johnson
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -31,7 +31,7 @@
 #include <stdlib.h>
 #include <guile/gh.h>
 
-#include <ctl-io.h>
+#include "ctl-io.h"
 #include <ctlgeom.h>
 
 /**************************************************************************/
